#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.illustration.interfaces import IIllustrationParagraph
from pyams_content.component.paragraph.interfaces import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph
from pyams_content_es.interfaces import IDocumentIndexInfo

# import packages
from pyams_utils.adapter import adapter_config
from pyams_utils.html import html_to_text


@adapter_config(context=IHeaderParagraph, provides=IDocumentIndexInfo)
def HeaderParagraphIndexInfo(paragraph):
    """Header paragraph index info"""
    info = {}
    for lang, header in paragraph.header.items():
        if header:
            info.setdefault(lang, header)
    return info


@adapter_config(context=IHTMLParagraph, provides=IDocumentIndexInfo)
def HTMLParagraphIndexInfo(paragraph):
    """HTML paragraph index info"""
    info = {}
    for lang, title in paragraph.title.items():
        if title:
            info.setdefault(lang, title)
    for lang, body in paragraph.body.items():
        if body:
            new_body = '{old}\n{body}'.format(old=info.get(lang, ''),
                                              body=html_to_text(body).replace('\r', ''))
            info[lang] = new_body
    return info


@adapter_config(context=IIllustrationParagraph, provides=IDocumentIndexInfo)
def IllustrationIndexInfo(paragraph):
    """Illustration index info"""
    info = {}
    for lang, title in paragraph.title.items():
        if title:
            info.setdefault(lang, title)
    for lang, legend in paragraph.alt_title.items():
        if legend:
            new_legend = '{old}\n{legend}'.format(old=info.get(lang, ''),
                                                  legend=legend)
            info[lang] = new_legend
    return info


@adapter_config(name='body', context=IParagraphContainerTarget, provides=IDocumentIndexInfo)
def ParagraphContainerTargetIndexInfo(content):
    """Paragraph container index info"""
    body = {}
    for paragraph in IParagraphContainer(content).values():
        info = IDocumentIndexInfo(paragraph, None)
        if info is not None:
            for lang, info_body in info.items():
                body[lang] = '{old}\n{body}'.format(old=body.get(lang, ''),
                                                    body=info_body)
    return {'body': body}
